package uniqueballs {

    import flash.display.MovieClip;
    import flash.display.Sprite;
    import flash.events.*;	
	import flash.text.TextField;
	import flash.text.TextFieldType;

	import UniqueControl;
	
    public class UniqueBalls extends MovieClip {
		
		private var myBall:Ball; // your ball and text field
		private var myBalls:Array = new Array();  // other people's balls
		
		private var myControl:UniqueControl;		
		private var myAppName:String = "uniqueballs" // change for different applications
		private var myGroupsOf:Number = 3; // max number per "room"
		private var myPersistence:Boolean = false; // true does not work on same computer (not really tested)
		private var myFill:Boolean = true; // new people fill up places people left
		// myMessage is an optional TextField to help in debugging
		
        public function UniqueBalls() {			
			
			myControl = new UniqueControl(myAppName, myGroupsOf, myPersistence, myFill, myMessage);
			
			// use myControl.setData(property, value); to set your values
			// use myControl.theirData.property; to get a list of other people's values for that property

			// connect means you have connected
			// added means that someone else has been added since you connected
			// removed means that someone else has been removed since you connected
			// change means that the other people's properties have changed
			myControl.addEventListener(Event.CONNECT, myControlConnected);
			myControl.addEventListener(Event.ADDED, myControlAdded);
			myControl.addEventListener(Event.REMOVED, myControlRemoved);						
			myControl.addEventListener(Event.CHANGE, myControlChange);
			
        }
		
		// this fires once connected 
		
		private function myControlConnected(e:Event) {			
			myMessage.appendText("connected\n");
			trace (e.target.error);	 // can error check based on this...				
			
			// Build other people's balls			
			if (myControl.theirData.x) { // others are here already
				// build the balls to match the myControl list of existing balls
				// when people leave, their spot is kept but turned to null
				// the null spots might be filled in later depending on myFill
				// myControl.theirData holds all the properties others have stored
				// myControl.theirData.x is a list of all their x properties
				// myControl.theirData.y is a list of all their y properties
				// myControl.theirData.text is a list of all their text values, etc.
				// these are your variables and names to make up and add
				var i:uint;
				for (i=0; i<myControl.theirData.x.length; i++) {
					if (myControl.theirData.x[i] == null) {
						myBalls.push(null);
					} else {
						myBalls.push(new Ball());
					}
					adjustBall(myBalls[myBalls.length-1]);					
				}
			}				
			
			// Build our ball
			myBall = new Ball();				
			myControl.setProperty("x", myBall.x = randomBetween(100,300));
			myControl.setProperty("y", myBall.y = randomBetween(100,200));				
			myControl.setProperty("text", "Type Your Name");
			myBall.textHolder.myText.text = "Type Your Name";
			addChild(myBall);
			myBall.myBackground.addEventListener(MouseEvent.MOUSE_DOWN, drag);
			myBall.myBackground.addEventListener(MouseEvent.MOUSE_UP, drop);
			myBall.textHolder.myText.addEventListener(Event.CHANGE, textChange);				
			
		}
		
		private function randomBetween(a:Number=10, b:Number=0) {
			return (Math.floor(Math.min(a,b) + Math.random() * Math.abs(a-b)));
		}
		
		// any time someone else's object changes this function gets called

		private function myControlChange(e:Event) {
			//myMessage.appendText("changed\n");			
			var i:uint;			
			for (i=0; i<myBalls.length; i++) {		
				if (myBalls[i] == null) {continue;}
				myBalls[i].visible = true;
				myBalls[i].x = myControl.theirData.x[i];
				myBalls[i].y = myControl.theirData.y[i];
				myBalls[i].textHolder.myText.text = myControl.theirData.text[i];			
			}			
		}				

		// these get called when someone else's object gets added or removed
		
		private function myControlAdded(e:Event) {						
			myMessage.appendText("added\n");
			
			var mySpot:Number;
			if (myFill) { // if we want to fill in spots that were left
				mySpot = myBalls.indexOf(null);
				if (mySpot == -1) {
					myBalls.push(new Ball());	
					mySpot = myBalls.length-1;
				} else {
					myBalls[mySpot] = new Ball();
				}											
			} else { // otherwise we just keep adding the balls to the end
				myBalls.push(new Ball());					
				mySpot = myBalls.length-1;
			}			
			adjustBall(myBalls[mySpot]);		
					
		}				
		
		private function adjustBall(b:Ball) {
			b.textHolder.myText.selectable = false;
			b.textHolder.myText.type = TextFieldType.DYNAMIC;
			b.visible = false; // just to avoid initial flicker
			b.alpha = .6;
			addChild(b);		
		}
		
		private function myControlRemoved(e:Event) {						
			myMessage.appendText("removed\n");
			var i:uint;
			if (myControl.theirData.x) { // if you are the last ball, theirData is empty
				for (i=0; i<=myControl.theirData.x.length; i++) {
					if (myControl.theirData.x[i] == null) {
						if (myBalls[i] != null) {
							removeChild(myBalls[i]);
							myBalls[i] = null;										
						}
					}
				}					
			} else { // remember that they do not necessarily close down in order
				for (i=0; i<myBalls.length; i++) {
					if (myBalls[i] != null) {
						removeChild(myBalls[i]);
						myBalls[i] = null;	
					}
				}
			}				
		}	
		
		// drag our object and record that it has changed
		
		private function drag(e:MouseEvent) {			
			myBall.startDrag();
			setChildIndex(myBall, numChildren-1);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, record);
		}
		
		private function drop(e:MouseEvent) {
			myBall.stopDrag();
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, record);
		}		
		
		private function record(e:MouseEvent) {
			myControl.setProperty("x", myBall.x);
			myControl.setProperty("y", myBall.y);
		}
		
		private function textChange(e:Event) {
			myControl.setProperty("text", e.target.text);
		}						
		
    }
} 